<?php

/**
 * @file
 * Contains \Drupal\easychart\Plugin\Field\FieldWidget\EasychartFormatter.
 */

namespace Drupal\easychart\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\easychart\EasychartHelper;

/**
 * Provides a default Easychart formatter.
 *
 * @FieldFormatter(
 *   id = "easychart_default",
 *   module = "easychart",
 *   label = @Translation("Default"),
 *   field_types = {
 *     "easychart"
 *   },
 *   quickedit = {
 *     "editor" = "disabled"
 *   }
 * )
 */
class Easychart extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {

    // Unique distinction by $entity_id & $delta.
    $entity_id = $items->getEntity()->id();
    $element = [];

    foreach ($items as $delta => $item) {
      if ($output = EasychartHelper::easychartPrintChart($item->getValue(), $entity_id, $delta)) {
        $element[$delta] = array('#markup' => $output['markup']);
        $element['#attached']['drupalSettings']['easychart'][$entity_id][$delta] = array(
          'config' => $output['config'],
          'csv' => $output['csv'],
        );
      }
    }

    if (!empty($element)) {
      EasychartHelper::addRenderLibraries($element);
    }

    return $element;
  }

}
